@@ -21,8 +21,8 @@ The file list may have files and/or directories. If a directory is specified,
 all .kid files found in the directory and any sub-directories are compiled.
 """
 
-__revision__ = "$Rev: 421 $"
-__date__ = "$Date: 2006-10-22 07:02:46 -0400 (Sun, 22 Oct 2006) $"
+__revision__ = "$Rev: 455 $"
+__date__ = "$Date: 2006-12-21 01:42:34 -0500 (Thu, 21 Dec 2006) $"
 __author__ = "Ryan Tomayko (rtomayko@gmail.com)"
 __copyright__ = "Copyright 2004-2005, Ryan Tomayko"
 __license__ = "MIT <http://www.opensource.org/licenses/mit-license.php>"
@@ -33,7 +33,7 @@ from getopt import getopt, GetoptError as gerror
 
 try:
     from os import EX_OK, EX_DATAERR, EX_USAGE
-except:
+except ImportError:
     EX_OK, EX_DATAERR, EX_USAGE = 0, 1, 2
 
 import kid.compiler
@@ -41,8 +41,9 @@ import kid.compiler
 def main():
     # get options
     try:
-        opts, args = getopt(sys.argv[1:], 'fshd=', ['force', 'source', 'help', 'strip-dest-dir='])
-    except gerror, msg:
+        opts, args = getopt(sys.argv[1:],
+            'fshd=', ['force', 'source', 'help', 'strip-dest-dir='])
+    except gerror, e:
         sys.stderr.write(str(e) + '\n')
         sys.stdout.write(__doc__)
         sys.exit(EX_USAGE)
@@ -67,26 +68,28 @@ def main():
 
     # a quick function for printing results
     def print_result(res):
-        stat, file = res
+        stat, filename = res
         if stat == True:
-            msg = 'compile: %s\n' % file
+            msg = 'compile: %s\n' % filename
         elif stat == False:
-            msg = 'fresh: %s\n' % file
+            msg = 'fresh: %s\n' % filename
         else:
-            msg = 'error: %s (%s)\n' % (file, stat)
+            msg = 'error: %s (%s)\n' % (filename, stat)
         sys.stderr.write(msg)
 
     # run through files and compile
     err = False
     for f in files:
         if isdir(f):
-            for res in kid.compiler.compile_dir(f, force=force, source=source, strip_dest_dir=strip_dest_dir):
+            for res in kid.compiler.compile_dir(f, force=force,
+                    source=source, strip_dest_dir=strip_dest_dir):
                 if res[0] not in (True, False):
                     err = True
                 print_result(res)
         else:
             try:
-                stat = kid.compiler.compile_file(f, force=force, source=source, strip_dest_dir=strip_dest_dir)
+                stat = kid.compiler.compile_file(f, force=force,
+                    source=source, strip_dest_dir=strip_dest_dir)
             except Exception, e:
                 stat, err = e, True
             print_result((stat, f))
