@@ -4,23 +4,21 @@ Compile XML to Python byte-code.
 
 """
 
-__revision__ = "$Rev: 421 $"
-__date__ = "$Date: 2006-10-22 07:02:46 -0400 (Sun, 22 Oct 2006) $"
+__revision__ = "$Rev: 460 $"
+__date__ = "$Date: 2007-01-07 04:04:12 -0500 (Sun, 07 Jan 2007) $"
 __author__ = "Ryan Tomayko (rtomayko@gmail.com)"
 __copyright__ = "Copyright 2004-2005, Ryan Tomayko"
 __license__ = "MIT <http://www.opensource.org/licenses/mit-license.php>"
 
-import sys
-import re
 import os
 import os.path
 import imp
 import stat
 import struct
 import marshal
-import new
 
 import kid
+from kid.codewriter import raise_template_error
 
 __all__ = ['KID_EXT', 'compile', 'compile_file', 'compile_dir']
 
@@ -57,6 +55,7 @@ def compile(source, filename='<string>', encoding=None, entity_map=None):
     py = kid.codewriter.parse(source, encoding, filename, entity_map)
     return py_compile(py, filename)
 
+
 _timestamp = lambda filename : os.stat(filename)[stat.ST_MTIME]
 
 class KidFile(object):
@@ -68,7 +67,7 @@ class KidFile(object):
         self.py_file = os.path.splitext(kid_file)[0] + '.py'
         self.strip_dest_dir = strip_dest_dir
         self.pyc_file = self.py_file + 'c'
-        self.encoding = encoding or 'utf-8'
+        self.encoding = encoding
         self.entity_map = entity_map
         fp = None
         if force:
@@ -101,16 +100,21 @@ class KidFile(object):
         return code
 
     def code(self):
+        """Get the compiled Python code for the template."""
         if self._code is None:
-            if not self.stale:
-                self._code = marshal.load(self._pyc_fp)
-            else:
+            if self.stale:
                 pyfile = self.py_file
                 if self.strip_dest_dir and \
                    self.py_file.startswith(self.strip_dest_dir):
                     pyfile = os.path.normpath(
                         self.py_file[len(self.strip_dest_dir):])
-                self._code = py_compile(self.python, pyfile)
+                try:
+                    self._code = py_compile(self.python, pyfile)
+                except Exception:
+                    raise_template_error(filename=self.kid_file,
+                        encoding=self.encoding)
+            else:
+                self._code = marshal.load(self._pyc_fp)
         return self._code
     code = property(code)
 
@@ -125,13 +129,13 @@ class KidFile(object):
 
     def dump_source(self, file=None):
         py = self.python
-        encoding = self.encoding
+        encoding = self.encoding or 'utf-8'
         file = file or self.py_file
         fp = _maybe_open(file, 'wb')
         if fp:
             try:
                 try:
-                     fp.write(py.encode(encoding))
+                    fp.write(py.encode(encoding))
                 finally:
                     fp.close()
             except IOError:
@@ -199,7 +203,7 @@ def compile_file(file, force=False, source=False, encoding=None,
     else:
         return False
 
-def compile_dir(dir, maxlevels=10, force=False, source=0,
+def compile_dir(dir, maxlevels=10, force=False, source=False,
         encoding=None, strip_dest_dir=None, entity_map=None):
     """Byte-compile all kid modules in the given directory tree.
 
@@ -219,8 +223,8 @@ def compile_dir(dir, maxlevels=10, force=False, source=0,
     for name in names:
         fullname = os.path.join(dir, name)
         if os.path.isfile(fullname):
-            head, tail = name[:-ext_len], name[-ext_len:]
-            if tail == KID_EXT:
+            ext = name[-ext_len:]
+            if ext == KID_EXT:
                 try:
                     stat = compile_file(fullname, force, source,
                         encoding, strip_dest_dir, entity_map)
@@ -229,7 +233,7 @@ def compile_dir(dir, maxlevels=10, force=False, source=0,
                     stat = e
                 yield stat, fullname
         elif maxlevels > 0 and name != os.curdir and name != os.pardir \
-                    and os.path.isdir(fullname) and not os.path.islink(fullname):
+                and os.path.isdir(fullname) and not os.path.islink(fullname):
             for res in compile_dir(fullname, maxlevels - 1, force, source,
                     encoding, strip_dest_dir, entity_map):
                 yield res
