@@ -1,3 +1,13 @@
+"""Namespace handling."""
+
+__revision__ = "$Rev: 454 $"
+__date__ = "$Date: 2006-12-20 23:36:51 -0500 (Wed, 20 Dec 2006) $"
+__author__ = "Ryan Tomayko (rtomayko@gmail.com)"
+__copyright__ = "Copyright 2004-2005, Ryan Tomayko"
+__license__ = "MIT <http://www.opensource.org/licenses/mit-license.php>"
+
+__all__ = ['Namespace', 'xml', 'xhtml', 'atom', 'rdf', 'rss', 'nons']
+
 
 namespaces = {}
 
@@ -46,5 +56,3 @@ atom = Namespace('http://purl.org/atom/ns#', 'atom')
 rdf = Namespace('http://www.w3.org/1999/02/22-rdf-syntax-ns#', 'rdf')
 rss = Namespace('http://purl.org/rss/1.0/', 'rss')
 nons = Namespace(None, None)
-
-__all__ = ['xml', 'xhtml', 'atom', 'rdf', 'rss', 'nons', 'Namespace']
