@@ -1,18 +1,22 @@
 """Pull-style interface for ElementTree."""
 
-__revision__ = "$Rev: 429 $"
-__date__ = "$Date: 2006-10-26 09:24:33 -0400 (Thu, 26 Oct 2006) $"
+__revision__ = "$Rev: 455 $"
+__date__ = "$Date: 2006-12-21 01:42:34 -0500 (Thu, 21 Dec 2006) $"
 __author__ = "Ryan Tomayko (rtomayko@gmail.com)"
 __copyright__ = "Copyright 2004-2005, Ryan Tomayko"
 __license__ = "MIT <http://www.opensource.org/licenses/mit-license.php>"
 
-from kid.et import *  # ElementTree
-from kid.util import open_resource, QuickTextReader, show_error
-
+import htmlentitydefs
 from xml.parsers import expat
 
+from kid.element import Element, Comment, ProcessingInstruction
+from kid.util import open_resource, QuickTextReader, get_expat_error
+
+__all__ = ['ElementStream', 'XML', 'document', 'Parser', 'ExpatParser',
+    'START', 'END', 'TEXT', 'COMMENT', 'PI', 'XML_DECL', 'DOCTYPE', 'LOCATION']
+
+
 # This is the default entity map:
-import htmlentitydefs
 default_entity_map = {}
 default_external_dtd = []
 for k, v in htmlentitydefs.name2codepoint.items():
@@ -21,20 +25,19 @@ for k, v in htmlentitydefs.name2codepoint.items():
 default_external_dtd = '\n'.join(default_external_dtd)
 
 
-# Bring common ElementTree objects into scope
 class InvalidStreamState(Exception):
     def __init__(self, msg="Invalid stream state."):
         Exception.__init__(self, msg)
 
 
 def XML(text, fragment=True, encoding=None, xmlns=None, entity_map=None):
-    """Element generator that reads from a string"""
+    """Convert XML string into an ElementStream."""
     if text.startswith('<?xml ') or text.startswith('<!DOCTYPE '):
         fragment = False
     if fragment:
         # allow XML fragments
         if xmlns:
-            text = '<xml xmlns="%s">%s</xml>' % (xmlns,text)
+            text = '<xml xmlns="%s">%s</xml>' % (xmlns, text)
         else:
             text = '<xml>%s</xml>' % text
         if isinstance(text, unicode):
@@ -50,7 +53,8 @@ def XML(text, fragment=True, encoding=None, xmlns=None, entity_map=None):
         return ElementStream(_coalesce(p, encoding=encoding))
 
 
-def document(file, encoding=None, filename=None, entity_map=None):
+def document(file, encoding=None, filename=None, entity_map=None, debug=False):
+    """Convert XML document into an Element stream."""
     if not hasattr(file, 'read'):
         if filename is None:
             filename = file
@@ -58,13 +62,12 @@ def document(file, encoding=None, filename=None, entity_map=None):
     else:
         if filename is None:
             filename = '<string>'
-    p = Parser(file, encoding, entity_map)
+    p = Parser(file, encoding, entity_map, debug)
     p._filename = filename
     return ElementStream(_coalesce(p, encoding=encoding))
 
 
 class ElementStream(object):
-
     """Provides a pull/streaming interface to ElementTree.
 
     Instances of this class are iterable. Most methods of the class act on
@@ -201,7 +204,7 @@ def to_unicode(value, encoding):
     return unicode(value, encoding)
 
 
-def _coalesce(stream, encoding, extended=True):
+def _coalesce(stream, encoding):
     """Coalesces TEXT events and namespace events.
 
     Fold multiple sequential TEXT events into a single event.
@@ -212,8 +215,6 @@ def _coalesce(stream, encoding, extended=True):
     textbuf = []
     namespaces = []
     last_ev = None
-    last = None
-    current = None
     stack = [None]
     for ev, item in stream:
         if ev == TEXT:
@@ -233,7 +234,7 @@ def _coalesce(stream, encoding, extended=True):
                 if prefix:
                     attrib['xmlns:%s' % prefix] = uri
                 else:
-                    attrib['xmlns'] =  uri
+                    attrib['xmlns'] = uri
             namespaces = []
             current = item
             stack.append(item)
@@ -241,7 +242,7 @@ def _coalesce(stream, encoding, extended=True):
             current = stack.pop()
         elif ev == START_NS:
             prefix, uri = item
-            namespaces.append( (prefix, uri) )
+            namespaces.append((prefix, uri))
             continue
         elif ev == END_NS:
             continue
@@ -267,13 +268,16 @@ END_NS = 11
 PI = 12
 COMMENT = 13
 
-
-def Parser(source, encoding=None, entity_map=None):
-    return ExpatParser(source, encoding, entity_map)
+# This is for forwarding the location in the XML document
+LOCATION = 20
 
 
-# Most of the following copied from ElementTree.XMLTreeBuilder
-# Differences from ET implementation:
+def Parser(source, encoding=None, entity_map=None, debug=False):
+    return ExpatParser(source, encoding, entity_map, debug)
+
+
+# Most of the following copied from ElementTree.XMLTreeBuilder.
+# Differences from ElementTree implementation:
 #
 #   * Specialized for generator based processing. Elements are built
 #     using a iterator approach instead of the TreeBuilder approach.
@@ -282,18 +286,30 @@ def Parser(source, encoding=None, entity_map=None):
 
 class ExpatParser(object):
 
-    def __init__(self, source, encoding=None, entity_map=None):
-        if not hasattr(source, 'read'):
+    def __init__(self, source, encoding=None, entity_map=None, debug=False):
+        if hasattr(source, 'read'):
+            filename = '<string>'
+        else:
             filename = source
             source = open(source, 'rb')
-        else:
-            filename = '<string>'
         self._filename = filename
         self._source = source
         self._encoding = encoding
         self._parser = parser = expat.ParserCreate(encoding, "}")
         self._queue = []
 
+        try:
+            self._parser.CurrentLineNumber
+        except AttributeError:
+            # Expat in Python < 2.4 does not provide the location,
+            # silently switch off the debug mode in this case
+            debug = False
+        if debug:
+            self.push = self._push_location
+        else:
+            self.push = self._push
+        self.debug = debug
+
         # callbacks
         parser.DefaultHandler = self._default
         parser.StartElementHandler = self._start
@@ -306,30 +322,21 @@ class ExpatParser(object):
         parser.XmlDeclHandler = self._xmldecl_handler
         parser.StartDoctypeDeclHandler = self._doctype_handler
 
-        # let expat do the buffering, if supported
-        try:
-            self._parser.buffer_text = True
-        except AttributeError:
-            pass
-        # use new-style attribute handling, if supported
-        try:
-            self._parser.ordered_attributes = True
-            self._parser.specified_attributes = True
-            parser.StartElementHandler = self._start_list
-        except AttributeError:
-            pass
+        # attributes
+        # (these should all become customizable at some point)
+        parser.buffer_text = True
+        parser.ordered_attributes = False
+        parser.specified_attributes = True
         self._doctype = None
-        # these should be come customizable at some point
         if entity_map:
             self.entity = entity_map
         else:
             self.entity = default_entity_map
         self.external_dtd = default_external_dtd
         # setup entity handling
-        self._parser.SetParamEntityParsing(
-            expat.XML_PARAM_ENTITY_PARSING_ALWAYS)
-        self._parser.ExternalEntityRefHandler = self._buildForeign
-        self._parser.UseForeignDTD()
+        parser.SetParamEntityParsing(expat.XML_PARAM_ENTITY_PARSING_ALWAYS)
+        parser.ExternalEntityRefHandler = self._buildForeign
+        parser.UseForeignDTD()
 
     def _buildForeign(self, context, base, systemId, publicId):
         import StringIO
@@ -340,66 +347,62 @@ class ExpatParser(object):
         self._parser = original_parser
         return True
 
-    def push(self, ev, stuff):
+    def _push(self, ev, stuff):
         self._queue.append((ev, stuff))
 
+    def _push_location(self, ev, stuff):
+        self._push(LOCATION, (self._parser.CurrentLineNumber,
+            self._parser.CurrentColumnNumber))
+        self._push(ev, stuff)
+
     def _expat_stream(self):
         bufsize = 4 * 1024 # 4K
         feed = self.feed
         read = self._source.read
-        done = False
-        while True:
-            while not done and len(self._queue) == 0:
+        more = True
+        while more:
+            while more and not self._queue:
                 data = read(bufsize)
                 if data == '':
                     self.close()
-                    done = True
+                    more = False
                 else:
                     feed(data)
-            for i in self._queue:
-                yield i
+            for item in self._queue:
+                yield item
             self._queue = []
-            if done:
-                break
 
     def __iter__(self):
-        names = {}
 
-        # XXX: hack to enable old namespace. This should be removed for 0.7
-        old_ns = 'http://naeblis.cx/ns/kid#'
-        new_ns = 'http://purl.org/kid/ns#'
-        def fixname(key):
-            if key.startswith(old_ns):
-                key = ''.join([new_ns, key[len(old_ns):]])
+        clarknames = {}
+        def clarkname(key):
             try:
-                name = names[key]
+                name = clarknames[key]
             except KeyError:
-                name = key
-                if "}" in name:
-                    name = "{" + name
-                names[key] = name
+                if "}" in key:
+                    name = "{" + key
+                else:
+                    name = key
+                clarknames[key] = name
             return name
 
         stack = []
-        parent = None
         current = None
-        for (ev, stuff) in self._expat_stream():
+        for ev, stuff in self._expat_stream():
             if ev == TEXT:
                 yield TEXT, stuff
             elif ev == START:
-                tag, attrib_in = stuff
-                tag = fixname(tag)
-                attrib = {}
-                if attrib_in:
-                    for key, value in attrib_in.items():
-                        attrib[fixname(key)] = value
+                tag, attrib = stuff
+                tag = clarkname(tag)
+                attrib = dict([(clarkname(key), value)
+                    for key, value in attrib.items()])
                 parent = current
                 current = Element(tag, attrib)
                 stack.append(current)
                 yield START, current
             elif ev == END:
                 current = stack.pop()
-                assert fixname(stuff) == current.tag
+                assert clarkname(stuff) == current.tag
                 parent = len(stack) and stack[-1] or None
                 yield END, current
             elif ev == COMMENT:
@@ -407,7 +410,7 @@ class ExpatParser(object):
                 yield START, current
                 yield END, current
             elif ev == PI:
-                current = ProcessingInstruction(stuff[0], stuff[1])
+                current = ProcessingInstruction(*stuff)
                 yield START, current
                 yield END, current
             else:
@@ -423,7 +426,7 @@ class ExpatParser(object):
                 e = 'Error parsing XML%s:\n%s%s' % (
                     e.filename and e.filename != '<string>'
                     and (' file %r' % e.filename) or '',
-                    show_error(e), str(e))
+                    get_expat_error(e), str(e))
             except Exception:
                 pass
             raise expat.ExpatError(e)
@@ -435,69 +438,43 @@ class ExpatParser(object):
             finally:
                 del self._parser # get rid of circular references
 
-    def _start(self, tag, attrib_in):
-        self._queue.append((START, (tag, attrib_in)))
-
-    def _start_list(self, tag, attrib_in):
-        attrib = None
-        if attrib_in:
-            attrib = {}
-            for i in range(0, len(attrib_in), 2):
-                attrib[attrib_in[i]] = attrib_in[i+1]
-        self._queue.append((START, (tag, attrib)))
+    def _start(self, tag, attrib):
+        self.push(START, (tag, attrib))
 
     def _data(self, text):
-        self._queue.append((TEXT, text))
+        self.push(TEXT, text)
 
     def _end(self, tag):
-        self._queue.append((END, tag))
+        self.push(END, tag)
 
     def _default(self, text):
-        prefix = text[:1]
-        if prefix == "&":
+        if text.startswith('&'):
             # deal with undefined entities
             try:
-                self._queue.append((TEXT, self.entity[text[1:-1]]))
+                self.push(TEXT, self.entity[text[1:-1]])
             except KeyError:
-                from xml.parsers import expat
-                raise expat.error(
-                    "undefined entity %s: line %d, column %d" %
-                    (text, self._parser.ErrorLineNumber,
-                    self._parser.ErrorColumnNumber)
-                    )
+                raise expat.error("undefined entity %s: line %d, column %d"
+                    % (text, self._parser.ErrorLineNumber,
+                        self._parser.ErrorColumnNumber))
         else:
             # XXX not sure what should happen here.
             # This gets: \n at the end of documents?, <![CDATA[, etc..
             pass
 
     def _pi(self, target, data):
-        self._queue.append((PI, (target, data)))
+        self.push(PI, (target, data))
 
     def _comment(self, text):
-        self._queue.append((COMMENT, text))
+        self.push(COMMENT, text)
 
     def _start_ns(self, prefix, uri):
-        # XXX: hack to enable backward compatibility for kid templates.
-        #      remove in version 0.7
-        if uri == 'http://naeblis.cx/ns/kid#':
-            newuri = 'http://purl.org/kid/ns#'
-            from warnings import warn
-            warn('Document uses old kid namespace [%s] this should be changed'
-                 ' to [%s].' % (uri, newuri))
-            uri = newuri
-        self._queue.append((START_NS, (prefix or '', uri)))
+        self.push(START_NS, (prefix or '', uri))
 
     def _end_ns(self, prefix):
-        self._queue.append((END_NS, prefix or ''))
+        self.push(END_NS, prefix or '')
 
     def _xmldecl_handler(self, version, encoding, standalone):
-        self._queue.append((XML_DECL, (version, encoding, standalone)))
+        self.push(XML_DECL, (version, encoding, standalone))
 
     def _doctype_handler(self, name, sysid, pubid, has_internal_subset):
-        self._queue.append((DOCTYPE, (name, pubid, sysid)))
-
-
-# Utilities
-
-__all__ = ['ElementStream', 'XML', 'document', 'Parser', 'ExpatParser',
-           'START', 'END', 'TEXT', 'COMMENT', 'PI', 'XML_DECL', 'DOCTYPE']
+        self.push(DOCTYPE, (name, pubid, sysid))
